/*
 * @license
 * Licensed Materials - Property of IBM
 * 5724-U18
 * Copyright IBM Corporation. 2016,2019
 */
  /*
The `maximo-chatbubble` element provides bubble shaped div for chatting service.

This element has chat message and date information on the bottom of the message. 

The 'clientviewable' checkbox can be displayed by setting the `show-client-viewable` attribute on the chatbubble.

Example:
```html
    <maximo-chatbubble 
    		additional-info="Demo" 
    		class="messageBubble" 
    		date="{{date}}"
    		datecal="simple"
    		direction="true"
    		label="{{message}}" 
    		maxwidth="500px"
    		on-clickimage="_showPersonInfo"
    		person-firstname="{{firstname}}" 
    		person-image="{{image}}" 
    		show-client-viewable>
    </maximo-chatbubble>
```

@demo
 */

Polymer({
	is: 'maximo-chatbubble',
	behaviors: [BaseComponent],
	listeners: {
	    'maximo-checkbox-tapped': '_viewableChanged',
	    'maximo-checkbox-swiped': '_viewableChanged'
	},
	properties: {
		/** Label that be shown in chat bubble */		
		label: {
			type: String,
			value : ''
		},
		/** Direction of point on bubble. left=true, right=false.  default is right	 */
		direction:{
			type: Boolean,
			value : false,
			notify: true,
			reflectToAttribute : true
		},
		/** Max width value for bubble. Width of bubble can not be longer than max width. */
		maxwidth:{
			type: String,
			value : '200px'
		},
		/** Label for date. if value is empty, this label will not be displayed. */
		date :{
			type: String,
			value: ''
		},
		/** Calculation method for date. simple or human	 */
		datecal :{
			type: String,
			value : '',
			notify: true
		},
		/** Image path */
		personImage :{
			type : String
		},
		/** First name of the person who wrote the message */
		personFirstname : {
			type : String
		},
		/** Aditional information about the person */
		additionalInfo : {
			type : Object
		},
		/** property set on element to determine whether to show client viewable checkbox */
		showClientViewable: {
			type: Boolean,
			value: false
		},
		/** log entry is viewable only by the client 
		 *  this is used to compute viewableChecked property
		 */
		clientViewable : {
			type : Boolean,
			value: false
		},
		/** sets the client viewable checkbox 'checked' */
		viewableChecked: {
			type: Boolean,
			computed: '_getViewableChecked(clientViewable)'
		},
		/** determines if the client viewable checkbox is visible */
		cbHidden: {
			type: 'string',
			computed: '_getCBHidden(showClientViewable,additionalInfo)'
		},
   		/** translated string for when clientViewable checkbox is checked */
   		viewableText: {
   			type: String,
   			value: ''	// value is set in attached
   		},
   		/** translated string for when clientViewable checkbox is unchecked */
   		nonviewableText: {
   			type: String,
   			value: ''	// value is set in attached
   		}
	},
	attached: function(){
		// load translated strings for clientViewable checkbox
		this.viewableText = $M.localize('uitext','mxapibase', 'viewable');
		this.nonviewableText = $M.localize('uitext','mxapibase', 'nonviewable');
	},
	getCommentDate : function(date){
		if(this.getAttribute('datecal')==='simple'){
			return this.calculateDate(date);
		}
		else{
			return this.calHumanreadableDate(date);
		}
	},
	calculateDate : function(date){
		var monthNames = ['January', 'February', 'March', 'April', 'May', 'June',
		                  'July', 'August', 'September', 'October', 'November', 'December'];
		
		var temp = Date.parse(date);
		var completionDate = new Date(temp);
		var month = monthNames[completionDate.getMonth()];
		var day = completionDate.getDate();
		var year = completionDate.getFullYear();
		return month+' '+day+' '+year;
	},
	
	calHumanreadableDate : function(date){
		var difference = '';
		var oneMinute=1000*60;
		var oneHour=oneMinute*60;
		var oneDay=oneHour*24;
		var oneMonth=oneDay*30;
		var oneYear = oneDay*365;
		
		if (date) {
			var msecs = Date.parse(date);
			var completionDate = new Date(msecs);
			var nowDate = new Date();
			
			var msecsDiff = nowDate.getTime() - completionDate.getTime();
			var value = 1;
			if(msecsDiff >= oneYear){
				return this.calculateDate(date);
			}
			else if (msecsDiff >= oneMonth) {
				value = Math.round(msecsDiff/oneMonth);								
				difference = (value > 1 ? this.localize('uitext', 'mxapibase', 'monthsago', [value.toString()]) : this.localize('uitext', 'mxapibase', 'monthago', [value.toString()]));						
			}
			else if (msecsDiff >= oneDay) {
				value = Math.round(msecsDiff/oneDay);
				difference = value > 1 ? this.localize('uitext', 'mxapibase', 'daysago', [value.toString()]) : this.localize('uitext', 'mxapibase', 'dayago', [value.toString()]);				
			}else if (msecsDiff >= oneHour){
				value = Math.round(msecsDiff/oneHour);
				difference = value > 1 ? this.localize('uitext', 'mxapibase', 'hoursago', [value.toString()]) : this.localize('uitext', 'mxapibase', 'hourago', [value.toString()]);				
			}else if (msecsDiff >= oneMinute) {
				value = Math.round(msecsDiff/oneMinute);
				difference = value > 1 ? this.localize('uitext', 'mxapibase', 'minutesago', [value.toString()]) : this.localize('uitext', 'mxapibase', 'minuteago', [value.toString()]);				
			}else {
				value = 1;
				difference =  this.localize('uitext', 'mxapibase', 'now');				
			}
		}		
		return difference;
	},
	_imgClicked : function(e) {
		this.fire('clickimage', e.detail);
	},
	/** fired by the client viewable checkbox */
	_viewableChanged : function(e) {
		// if the value of the internalExternal checkbox changes, update clientviewable and label
		this.clientViewable = e.detail.checked;
		this.fire('client-viewable-changed', this);
	},
	_getViewableChecked: function(viewable) {
		if (viewable){
			return true;
		} else {
			return false;
		}
	},
	/** can only show the viewable checkbox if the current user is the user on the comment and it's not a BotC session */
	_showViewableCheckbox: function(){
		if (this.showClientViewable && this.additionalInfo && this.additionalInfo.personid && this.additionalInfo.personid === $M.userInfo.personid){
			return true;
		}
		return false;
	},
	/** get the class for the outer div (flex or flex-end-justified), and specify whether it's right (left is default) */
	_getFlexClass: function(direction){
		if (direction){
			return 'flex';
		}
		return 'flex-end-justified right';
	},
	_getCBHidden: function(){
		if (this._showViewableCheckbox()){
			return false;
		}
		return true;
	},


});